@@ -69,69 +69,54 @@ class ListOf(list):
             values,
             type_: Union[type, tuple] = UNDEF,
             validate: Callable = None):
+
+        # Normailize type_ to be instance of `tuple`
         if isinstance(type_, type):
             self.types = (type_,)
         else:
             self.types = type_
-        self.validate = validate
+            
+        self.validate_func = validate
 
-        errors = list()
         if self.types[0] == UNDEF:
             return super().__init__(values)
-        for value in values:
-            if isinstance(value, dict):
-                dict_error = False
-                model_types = []
-                for type_ in self.types:
-                    if isinstance(type_, Model):
-                        model_types.append(type_)
-                        try:
-                            type_(value)
-                            dict_error = False
-                        except Exception as e:
-                            dict_error = True
-                if dict_error == True:
-                    errors.append(
-                        f"Data doesn't comply with either {model_types}"
-                    )
-            else:
-                try:
-                    assert isinstance(value, self.types),\
-                        f"'{value}' is not instance of {self.types}"
-                except Exception as e:
-                    errors.append(e)
-            if callable(self.validate):
-                try:
-                    self.validate(value)
-                except Exception as e:
-                    errors.append(e)
 
-        if errors:
-            raise ListOf.ValueError(errors)
+        for value in values:
+            self._validate(value)
+    
         return super().__init__(values)
 
     def __setitem__(self, index, value):
         """Set list value at ``index`` if ``value`` is valid"""
-        if self.types is not UNDEF:
-            assert isinstance(value, self.types),\
-                f"'{value}' is not instance of {self.types}"
-        if callable(self.validate):
-            self.validate(value)
 
+        self._validate(value)
         return super().__setitem__(index, value)
 
-    def append(self, value):
-        """Append object to the end of the list if ``value`` is valid."""
-        if self.types[0] is not UNDEF:
-            assert isinstance(value, self.types),\
-                f"'{value}' is not instance of {self.types}"
-        if callable(self.validate):
-            self.validate(value)
+    def _validate(self, value):
+        if self.types[0] == UNDEF:
+            return
+        if isinstance(value, dict):
+            models = filter(lambda type_: isinstance(type_, Model), self.types)
+            for model_cls in models:
+                try:
+                    model_cls(value)
+                    return
+                except: pass
+        assert isinstance(value, self.types),\
+            f"'{value}' is not instance of {self.types}"
 
+        if callable(self.validate_func):
+            self.validate_func(value)
+
+    def append(self, value):
+        """Append object to the list if ``value`` is valid."""
+
+        self._validate(value)
         return super().append(value)
 
     def list(self):
         """Return data as native `list`"""
+
         data = []
         for item in self:
             if isinstance(item, Model):
